from django.db.models import Manager
from django.db.models.query_utils import Q

class PermissionManager(Manager):

    def for_user(self, user=None):
        if user and user.is_authenticated():
            return self.get_query_set().filter(Q(group=None) | Q(group__user=user))
        else:
            return self.get_query_set().filter(group=None)



from django.conf import settings
from django.http import HttpResponseForbidden
from django.template import RequestContext,Template,loader,TemplateDoesNotExist
from django.utils.importlib import import_module

"""
# Middleware to allow the display of a 403.html template when a
# 403 error is raised.
"""

class Http403(Exception):
    pass

class Http403Middleware(object):
    def process_exception(self, request, exception):
        from http import Http403

        if not isinstance(exception, Http403):
            # Return None so django doesn't re-raise the exception
            return None

        try:
            # Handle import error but allow any type error from view
            callback = getattr(import_module(settings.ROOT_URLCONF),'handler403')
            return callback(request,exception)
        except (ImportError,AttributeError):
            # Try to get a 403 template
            try:
                # First look for a user-defined template named "403.html"
                t = loader.get_template('403.html')
            except TemplateDoesNotExist:
                # If a template doesn't exist in the projct, use the following hardcoded template
                t = Template("""{% load i18n %}
                 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
                        "http://www.w3.org/TR/html4/strict.dtd">
                 <html>
                 <head>
                     <title>{% trans "403 ERROR: Access denied" %}</title>
                 </head>
                 <body>
                     <h1>{% trans "Access Denied (403)" %}</h1>
                     {% trans "We're sorry, but you are not authorized to view this page." %}
                 </body>
                 </html>""")

            # Now use context and render template
            c = RequestContext(request, {
                  'message': exception.message
             })

            return HttpResponseForbidden(t.render(c))


# encoding: utf-8

from BeautifulSoup import BeautifulSoup
from markdown import markdown as original_markdown
from unidecode import unidecode

import re

def markdown(content, *args, **kwargs):
    """
    Converts markdown and makes it safe html with Beautiful Soup.
    """

    marked = original_markdown(content, ['def_list', 'codehilite'], *args, **kwargs)
    return unicode(BeautifulSoup(marked, fromEncoding="utf-8"))

def slugify(value):
    """
    Normalizes a string using unidecode library. Lowercase it, remove
    punctuation and replace spacing with hyphens.

    >>> slugify(u'Blåbærsyltetøy')
    'Blabaersyltetoy'
    """
    value = unidecode(value)
    value = unicode(re.sub('[^\w\s-]', '', value).strip().lower())
    return re.sub('[-\s]+', '-', value)






import floppyforms as forms

class MarkdownTextarea(forms.Textarea):
    template_name = 'samklang_utils/markdown_textarea.html'

    class Media:
        js = ('js/wmd.js', 'js/flext.js')
        css = {
                'all': ('css/wmd.css',)
                }

    def get_context_data(self):
        self.attrs['class'] += " flext growme maxheight-500"
        return super(MarkdownTextarea, self).get_context_data()

class SharedToGroup(forms.MultiWidget):
    #template_name = 'samklang_utils/shared_to_group.html'

    def __init__(self, attrs=None):
        widgets = (forms.Checkbox, forms.Modelselect)
        super(SharedToGroup, self).__init__(widgets, attrs)




class LazySite(object):
    def __get__(self, request, obj_type=None):
        if not hasattr(request, '_cached_site'):
            from django.contrib.sites.models import Site
            request._cached_site = Site.objects.get(domain=request.get_host().split(":")[0].lower())
        return request._cached_site

class SiteMiddleware(object):
    def process_request(self, request):
        request.__class__.site = LazySite()
        return None




#!/usr/bin/env python
from distutils.core import setup

setup(
        name='samklang-utils',
        version="0.3.1",
        author='Sigurd Gartmann',
        author_email='sigurdga-samklang@sigurdga.no',
        url='http://github.com/sigurdga/samklang-utils',
        description='Common utilities for Samklang',
        long_description=open('README.txt').read(),
        license="AGPL",
        packages = ['samklang_utils', 'samklang_utils.middleware'],
        package_data = {'samklang_utils': ['static/js/*.js', 'static/css/*.css', 'locale/*/LC_MESSAGES/django.*o'] },
        install_requires=['django-floppyforms>=0.4.7'],
        classifiers=[
                "Development Status :: 3 - Alpha",
                "License :: OSI Approved :: GNU Affero General Public License v3",
                "Intended Audience :: Developers",
                "Framework :: Django",
                "Environment :: Web Environment",
                "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
                ]
        )

